<?php
/**
 * Copyright (C) 2019 Leipzig University Library
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @author  Sebastian Kehr <kehr@ub.uni-leipzig.de>
 * @license http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 */

use fid\FormModel\AddressValidator;
use Zend\Filter\Boolean;
use Zend\Filter\StringTrim;
use Zend\Filter\ToInt;
use Zend\Filter\ToNull;
use Zend\Form\Element\Checkbox;
use Zend\Form\Element\Number;
use Zend\Form\Element\Password;
use Zend\Form\Element\Radio;
use Zend\Form\Element\Select;
use Zend\Form\Element\Submit;
use Zend\Form\Element\Text;
use Zend\Hydrator\ClassMethods;
use Zend\Validator\Callback;
use Zend\Validator\EmailAddress;
use Zend\Validator\Identical;
use Zend\Validator\NotEmpty;
use Zend\Validator\Regex;
use Zend\Validator\StringLength;

return [
    'hydrator'     => ClassMethods::class,
    'elements'     => [
        'username'             => [
            'spec' => [
                'name'       => 'username',
                'type'       => Text::class,
                'options'    => [
                    'label' => 'label_username',
                ],
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'password'             => [
            'spec' => [
                'name'       => 'password',
                'type'       => Password::class,
                'options'    => [
                    'label' => 'label_password',
                ],
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'passwordConfirmation' => [
            'spec' => [
                'name'       => 'passwordConfirmation',
                'type'       => Password::class,
                'options'    => [
                    'label' => 'label_password_confirmation',
                ],
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'salutation'           => [
            'spec' => [
                'name'    => 'salutation',
                'type'    => Select::class,
                'options' => [
                    'label'        => 'label_salutation',
                    'options'      => [
                        'mr'  => [
                            'value' => 'mr',
                            'label' => 'label_salutation_mr',
                        ],
                        'mrs' => [
                            'value' => 'mrs',
                            'label' => 'label_salutation_mrs',
                        ]
                    ],
                    'empty_option' => '',
                ],
            ],
        ],
        'academicTitle'        => [
            'spec' => [
                'name'    => 'academicTitle',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_academic_title',
                ],
            ],
        ],
        'firstname'            => [
            'spec' => [
                'name'       => 'firstname',
                'type'       => Text::class,
                'options'    => [
                    'label' => 'label_firstname',
                ],
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'lastname'             => [
            'spec' => [
                'name'       => 'lastname',
                'type'       => Text::class,
                'options'    => [
                    'label' => 'label_lastname',
                ],
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'homeLibrary'          => [
            'spec' => [
                'name'       => 'homeLibrary',
                'type'       => Select::class,
                'options'    => [
                    'label'        => 'label_home_library',
                    'empty_option' => '',
                ],
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'accessLevel'          => [
            'spec' => [
                'name'       => 'accessLevel',
                'type'       => Radio::class,
                'options'    => [
                    'label' => 'label_access_level',
                ],
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'yearOfBirth'          => [
            'spec' => [
                'name'       => 'yearOfBirth',
                'type'       => Number::class,
                'options'    => [
                    'label' => 'label_year_of_birth',
                ],
                'attributes' => [
                    'min' => 1900,
                    'max' => 2018,
                ],
            ]
        ],
        'jobTitle'             => [
            'spec' => [
                'name'    => 'jobTitle',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_job_title',
                ],
            ],
        ],
        'college'              => [
            'spec' => [
                'name'    => 'college',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_college',
                ]
            ],
        ],
        'addressLine1'         => [
            'spec' => [
                'name'    => 'addressLine1',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_address_line_1',
                ]
            ],
        ],
        'addressLine2'         => [
            'spec' => [
                'name'    => 'addressLine2',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_address_line_2',
                ]
            ],
        ],
        'addressZip'         => [
            'spec' => [
                'name'    => 'addressZip',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_address_zip',
                ]
            ],
        ],
        'addressCity'         => [
            'spec' => [
                'name'    => 'addressCity',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_address_city',
                ]
            ],
        ],
        'addressCountry'         => [
            'spec' => [
                'name'    => 'addressCountry',
                'type'    => Text::class,
                'options' => [
                    'label' => 'label_address_country',
                ]
            ],
        ],
        'eulaAccepted'         => [
            'spec' => [
                'name'       => 'eulaAccepted',
                'type'       => Checkbox::class,
                'attributes' => [
                    'required' => 'required',
                ],
            ],
        ],
        'submit'               => [
            'spec' => [
                'name'       => 'submit',
                'type'       => Submit::class,
                'attributes' => [
                    'value' => 'label_submit',
                ],
            ],
        ],
    ],
    'input_filter' => [
        'username'             => [
            'name'       => 'username',
            'required'   => true,
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ],
                ],
                EmailAddress::class => [
                    'name' => EmailAddress::class,
                ],
            ],
        ],
        'password'             => [
            'name'       => 'password',
            'required'   => true,
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'min'      => 8,
                        'max'      => 255,
                        'messages' => [
                            StringLength::TOO_SHORT => 'error_password_length',
                            StringLength::TOO_LONG  => 'error_password_length',
                        ],
                    ],
                ],
                Regex::class        => [
                    'name'    => Regex::class,
                    'options' => [
                        'pattern'  => '/(?:.*[0-9].*[^\w].*|.*[^\w].*[0-9].*)/',
                        'messages' => [
                            Regex::NOT_MATCH => 'error_password_pattern',
                        ],
                    ],
                ],
                Identical::class    => [
                    'name'    => Identical::class,
                    'options' => [
                        'strict'   => false,
                        'token'    => 'passwordConfirmation',
                        'messages' => [
                            Identical::NOT_SAME => 'error_password_confirmation',
                        ],
                    ],
                ],
            ],
        ],
        'passwordConfirmation' => [
            'name'       => 'passwordConfirmation',
            'required'   => true,
            'validators' => [
                NotEmpty::class => [
                    'name' => NotEmpty::class,
                ],
            ],
        ],
        'salutation'           => [
            'name'     => 'salutation',
            'required' => false,
            'filters'  => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
        ],
        'academicTitle'        => [
            'name'     => 'academicTitle',
            'required' => false,
            'filters'  => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'firstname'            => [
            'name'       => 'firstname',
            'required'   => true,
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ],
                ],
                Regex::class        => [
                    'name'    => Regex::class,
                    'options' => [
                        'pattern' => '/^\D*$/',
                    ],
                ],
            ],
        ],
        'lastname'             => [
            'name'       => 'lastname',
            'required'   => true,
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
                Regex::class        => [
                    'name'    => Regex::class,
                    'options' => [
                        'pattern' => '/^\D*$/',
                    ],
                ],
            ],
        ],
        'homeLibrary'          => [
            'name'     => 'homeLibrary',
            'required' => true,
            'filters'  => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'accessLevel'          => [
            'name'     => 'accessLevel',
            'required' => true,
            'filters'  => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
        ],
        'yearOfBirth'          => [
            'name'     => 'yearOfBirth',
            'required' => false,
            'filters'  => [
                ToNull::class => [
                    'name' => ToNull::class,
                ],
                ToInt::class  => [
                    'name' => ToInt::class,
                ],
            ],
        ],
        'jobTitle'             => [
            'name'     => 'jobTitle',
            'required' => false,
            'filters'  => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'college'              => [
            'name'     => 'college',
            'required' => false,
            'filters'  => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'addressLine1'         => [
            'name'       => 'addressLine1',
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                NotEmpty::class => [
                    'name'    => NotEmpty::class,
                    'options' => [
                        'type' => NotEmpty::NULL,
                    ],
                ],
                Callback::class => [
                    'name'    => Callback::class,
                    'options' => [
                        'callback' => AddressValidator::class.'::init',
                        'messages' => [
                            Callback::INVALID_VALUE => 'error_empty_address_value',
                        ],
                    ],
                ],
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'addressLine2' => [
            'name'       => 'addressLine2',
            'required'   => false,
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'addressZip' => [
            'name'       => 'addressZip',
            'required'   => false,
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                NotEmpty::class => [
                    'name'    => NotEmpty::class,
                    'options' => [
                        'type' => NotEmpty::NULL,
                    ],
                ],
                Callback::class => [
                    'name'    => Callback::class,
                    'options' => [
                        'callback' => AddressValidator::class.'::init',
                        'messages' => [
                            Callback::INVALID_VALUE => 'error_empty_address_value',
                        ],
                    ],
                ],
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'addressCity' => [
            'name'       => 'addressCity',
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                NotEmpty::class => [
                    'name'    => NotEmpty::class,
                    'options' => [
                        'type' => NotEmpty::NULL,
                    ],
                ],
                Callback::class => [
                    'name'    => Callback::class,
                    'options' => [
                        'callback' => AddressValidator::class.'::init',
                        'messages' => [
                            Callback::INVALID_VALUE => 'error_empty_address_value',
                        ],
                    ],
                ],
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'addressCountry' => [
            'name'       => 'addressCountry',
            'filters'    => [
                StringTrim::class => [
                    'name' => StringTrim::class,
                ],
            ],
            'validators' => [
                NotEmpty::class => [
                    'name'    => NotEmpty::class,
                    'options' => [
                        'type' => NotEmpty::NULL,
                    ],
                ],
                Callback::class => [
                    'name'    => Callback::class,
                    'options' => [
                        'callback' => AddressValidator::class.'::init',
                        'messages' => [
                            Callback::INVALID_VALUE => 'error_empty_address_value',
                        ],
                    ],
                ],
                StringLength::class => [
                    'name'    => StringLength::class,
                    'options' => [
                        'max' => 255
                    ]
                ],
            ],
        ],
        'eulaAccepted'         => [
            'name'       => 'eulaAccepted',
            'required'   => true,
            'filters'    => [
                Boolean::class => [
                    'name' => Boolean::class,
                ],
            ],
            'validators' => [
                NotEmpty::class => [
                    'name'    => NotEmpty::class,
                    'options' => [
                        'messages' => [
                            NotEmpty::IS_EMPTY => 'error_eula_accepted',
                        ],
                    ],
                ],
            ],
        ],
        'submit'               => [
            'name'     => 'submit',
            'required' => true,
        ],
    ],
];
